/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is forcect
 *
 * The Initial Developer of the Original Code is 
 * Ryan Gammon <rggammon@alumni.uwaterloo.ca>
 * Portions created by Ryan Gammon are Copyright (C) 2003
 * Ryan Gammon. All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

const FORCECT_CID = Components.ID('{5f9369d7-bfd0-4530-9417-2637393b29ae}');
const FORCECT_CONTRACTID = "@mozilla.org/js_forcect;1";
const NETMODULEMGR_CONTRACTID = "@mozilla.org/network/net-extern-mod;1";
const LOADER_CONTRACTID = "@mozilla.org/moz/jssubscript-loader;1";
const CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";
const PREF_CONTRACTID = "@mozilla.org/preferences;1";

// The list of available CONTRACTIDS to register for notification on.
// From mozilla/netwerk/base/public/nsINetModuleMgr.idl
const NS_NETWORK_MODULE_MANAGER_HTTP_REQUEST_CONTRACTID  = "@mozilla.org/network/moduleMgr/http/request;1";
const NS_NETWORK_MODULE_MANAGER_HTTP_RESPONSE_CONTRACTID = "@mozilla.org/network/moduleMgr/http/response;1";

// Our preferences
const ENABLE_ON_STARTUP_PREF = "forcect.enableOnStartup";

function nsForceContentTypeService() {
}

var nsForceContentTypeService = {
    
    // Begin nsIHttpNotify implementation
    onModifyRequest : function (oHttp) {
        // Never called, as expected (I don't register ...HTTP_REQUEST...)
    },

    onExamineResponse : function (oHttp) {
        if(this.filters) {
            for(var i = 0; i < this.filters.length; i++) {
                //dump("Matching against " + this.filters[i].uriRegexp + "\n");
                //dump(oHttp.URI.spec + " " + oHttp.contentType + "\n");
                if(this.filters[i].uriRegexp.test(oHttp.URI.spec) &&
                   this.filters[i].oldCTRegexp.test(oHttp.contentType)) {
                       dump("Changing Content-Type to " + this.filters[i].newCT + "\n");
                       oHttp.contentType = this.filters[i].newCT;
                       break;
                }
            }
        }
    },
    // End nsIHttpNotify implementation

    // Begin nsIHttpHeaderVisitor implementation
    visitHeader : function (aHeader, aValue) {
        // Useful for debugging...
        //   oHttp.QueryInterface(Components.interfaces.nsIRequest);
        //   oHttp.visitRequestHeaders(this);        
        //   oHttp.visitResponseHeaders(this);        

        dump("Got header: " + aHeader + " " + aValue + "\n");
    },
    // End nsIHttpHeaderVisitor implementation

    // Begin nsIObserver implementation
    observe: function(aSubject, aTopic, aData) {
        if (aTopic == 'app-startup') {
            // Load jslib. 
            // XXX: We could actually do this in something like loadRdf() instead of
            // using an app-startup listener (I had it that way originally, but moved
            // it here while debugging an rdf problem [missing the sync flag])
            var jsLoader = Components.classes[LOADER_CONTRACTID].createInstance(Components.interfaces.mozIJSSubScriptLoader);  
            jsLoader.loadSubScript("chrome://jslib/content/jslib.js");
        
            // include function is defined by jslib, which we just loaded
            include('chrome://jslib/content/io/dirUtils.js');
            include('chrome://jslib/content/io/dir.js');
            include('chrome://jslib/content/rdf/rdf.js');
            include('chrome://jslib/content/rdf/rdfFile.js');
        } 
        else {
            dump("nsForceContentTypeService: unknown TOPIC: " + aTopic + "\n");
        }
    },
    // End nsIObserver implementation

    // Begin nsIForceContentTypeService implementation
    isEnabled : false, // Exposed as a readonly attribute
    filters : null,
    isInitialized: false,

    init: function() {
        if(!this.isInitialized) {
            var prefs = Components.classes[PREF_CONTRACTID].getService();
            prefs.QueryInterface(Components.interfaces.nsIPref);
            if(prefs.GetPrefType(ENABLE_ON_STARTUP_PREF) == prefs.ePrefInvalid) {
                dump("Setting pref (init)\n");
                prefs.SetBoolPref(ENABLE_ON_STARTUP_PREF, false);
            }
            else {
                if(prefs.GetBoolPref(ENABLE_ON_STARTUP_PREF)) {
                    this.enable();
                }
            }            
            this.isInitialized = true;
        }
    },

    enable: function() {
        // Register listener
        if(!this.isEnabled) {
            // Get config information
            this.loadRdf();

            var netModuleMgr = Components.classes[NETMODULEMGR_CONTRACTID].getService(Components.interfaces.nsINetModuleMgr);
            netModuleMgr.registerModule(NS_NETWORK_MODULE_MANAGER_HTTP_RESPONSE_CONTRACTID, nsForceContentTypeService);
            this.isEnabled = true;
            dump("Enabled forcect service\n");
        }
    },

    disable: function() {
        if(this.isEnabled) {
            var netModuleMgr = Components.classes[NETMODULEMGR_CONTRACTID].getService(Components.interfaces.nsINetModuleMgr);
            netModuleMgr.unregisterModule(NS_NETWORK_MODULE_MANAGER_HTTP_RESPONSE_CONTRACTID, nsForceContentTypeService);
            this.isEnabled = false;
            dump("Disabled forcect service\n");
        }
    },

    loadRdf: function() {
        this.filters = new Array();
                
        var utils = new DirUtils();

        var profileDir = new Dir(utils.getMozUserHomeDir());
        var file = profileDir.append("forcect.rdf");

        rdfFile = new RDFFile(file, JS_RDFFILE_FLAG_DONT_CREATE | JS_RDFFILE_FLAG_SYNC);
        if(rdfFile) {
            var seq = rdfFile.getRootSeq("urn:ForceContentTypeRoot");

            var nodes = seq.getSubNodes();
            for(var i = 0; i < nodes.length; i++) {
                var filter = { uriRegexp: null, oldCTRegexp: null, newCT: null };
                filter.uriRegexp   = new RegExp(nodes[i].getAttribute("http://forcect.mozdev.org/rdf#URIRegexp"));
                filter.oldCTRegexp = new RegExp(nodes[i].getAttribute("http://forcect.mozdev.org/rdf#OldCTRegexp"));
                filter.newCT       = nodes[i].getAttribute("http://forcect.mozdev.org/rdf#NewCT");

                dump("Adding " + filter.uriRegexp + " " + filter.oldCTRegexp + " " + filter.newCT + "\n");

                this.filters.push(filter);
            }
        }
    },

    // End nsIForceContentTypeService implementation

    // Begin nsISupports implementation
    QueryInterface: function(iid) {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.nsIHttpHeaderVisitor) &&
            !iid.equals(Components.interfaces.nsIHttpNotify) &&
            !iid.equals(Components.interfaces.nsIForceContentTypeService) &&
            !iid.equals(Components.interfaces.nsIObserver)) {
                // dump("forcectService: QI unknown interface: " + iid + "\n");
                throw Components.results.NS_ERROR_NO_INTERFACE;
            }
        return this;
    }
    // End nsISupports implementation
};

// nsForceContentType Class Factory
var nsForceContentTypeFactory = {
    createInstance: function(outer, iid) {
        if (outer != null) {
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        }
    
        // XXX: Is it cool to throw NS_ERROR_NO_INTERFACE via QI on failure?
        // Some components check iid's here, and return NS_ERROR_INVALID_ARG instead
        return nsForceContentTypeService.QueryInterface(iid);
    }
}


// nsForceContentType Module (for XPCOM registration) 
var nsForceContentTypeModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        // We don't need to defer registration (as seen in nsSample.js)

        // dump("Registering forcect\n");
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(FORCECT_CID,
                                        "Force Content-Type JS component",
                                        FORCECT_CONTRACTID,
                                        fileSpec, location, type);


        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.addCategoryEntry("app-startup", "ForceCT", FORCECT_CONTRACTID, true, true);
    },

    unregisterSelf: function(compMgr, fileSpec, location) {
        // When does unregister ever get called, anyway?
        dump("UNREGISTERING forcect\n");
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(FORCECT_CID, fileSpec);

        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.deleteCategoryEntry("app-startup", FORCECT_CONTRACTID, true);
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(FORCECT_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return nsForceContentTypeFactory;
    },

    canUnload: function(compMgr) { 
        return true; 
    }
};

/* module initialization */
function NSGetModule(comMgr, fileSpec) { 
    return nsForceContentTypeModule; 
}

